#ifndef __DATE_H__
#define __DATE_H__

#include <iostream>
#include <fstream>
using namespace std;

/**
 * Classe representant une date, de la forme jour/mois/annee.
 */
class Date {

// ATTRIBUTS DE LA CLASSE
private:
    /** Entier rempresentant le jour courant (de 1 a 31). */
    int _jour;
    /** Entier rempresentant le mois courant (de 1 a 12). */
    int _mois;
    /** Entier rempresentant l'annee courante. */
    int _annee;

// METHODES PRIVEES
    /**
     * Verifie que la date courante est valide
     *@return true si la date courante est valide
     */
    /* Note : cette méthode ne pourra être appelée que de la classe Date (par d'autres méthodes de la classe Date.
    Ca n'aurait d'ailleurs pas de sens d'appeler cette mÃ©thode du main (par exe) puisque que l'on s'assurera que toute Date est valide Ã  tout moment */
    bool estValide() const;

// METHODES PUBLIQUES


public:
    //-----------------------------
    // Constructeurs/destructeurs
    //-----------------------------
    /** Constructeur par defaut : cree une nouvelle date (01/01/0000). */
    Date();

    /**
     * Constructeur par initialisation : cree une nouvelle date (iJour/iMois/iAnnee).
     *@param iJour entier representant le jour
     *@param iMois entier representant le mois
     *@param iAnnee entier representant l'annee
     */
    Date(int iJour, int iMois, int iAnnee);

    /// constructeur par copie
    Date(const Date & iDate);

    /** Destructeur. */
    ~Date();

    //-----------------------------
    // Selecteurs
    //-----------------------------
    /** Retourne le jour courant. */
    int getJour() const;
    /** Retourne le mois courant. */
    int getMois() const;
    /** Retourne l'annee courante. */
    int getAnnee() const;
    /** Modifie le jour courant. */
    void setJour(int iJour);
    /** Modifie le mois courant. */
    void setMois(int iMois);
    /** Modifie l'annee courante. */
    void setAnnee(int iAnnee);

    //-----------------------------
    // Methodes
    //-----------------------------
    /** output de enfant (par defaut dans la sortie standard) */
    virtual void output(ostream & flux=cout) const;
    /** inout de enfant (par defaut via le clavier) */
    virtual void input(istream & flux=cin);



    //-----------------------------
    // Surcharges d'operateurs
    //-----------------------------
    /** Operateur d'affectation. */
    Date & operator = (const Date & d);
    /** Teste si la date courante est strictement inferieure a la date d donnee en parametre. */
    bool operator < (const Date& d) const;
    /** Teste si la date courante est strictement superieure a la date d donnee en parametre. */
    bool operator > (const Date& d) const;
    /** Teste si la date courante est inferieure ou egale a la date d donnee en parametre. */
    bool operator <= (const Date& d) const;
    /** Teste si la date courante est superieure ou egale a la date d donnee en parametre. */
    bool operator >= (const Date& d) const;
    /** Teste si la date courante est egale a la date d donnee en parametre. */
    bool operator == (const Date& d) const;
    /** Teste si la date courante est differente de la date d donnee en parametre. */
    bool operator != (const Date& d) const;

    // operateurs de flux
    //--------------------
    /** Lecture de la date courante dans un flux. */
    friend istream& operator >> (istream& is, Date& d);
    /** Ecriture de la date courante dans un flux. */
    friend ostream& operator << (ostream& os, const Date& d);

    /** Lecture de la date ifstream*/
    friend ifstream& operator >> (ifstream& is, Date& d);
    /** Ecriture de la date ofstream */
    friend ofstream& operator << (ofstream& os, const Date& d);

};
    //----------------------
    // Auters Fonctions (ce ne sont pas des methodes de la classe Date)
    //----------------------
    /** Teste si une annee est bissextile. */
    bool isAnneeBissextile(int iAnnee);
    /** Calcule le nombre de jour dans un mois donne, pour une annee donnee. */
    int nbJoursDuMois(int iMois, int iAnnee);

#endif // ifndef __DATE_H__

