#ifndef __PERSONNE_H__
#define __PERSONNE_H__

#include <iostream>
#include <fstream>

#include "date.h"


/**
 * Classe representant une personne: nom, prenom et date de naissance.
 */
class Personne {

// ATTRIBUTS DE LA CLASSE
protected:
    /** Le prenom de la personne. */
    string _prenom;
    /** Le nom de la personne. */
    string _nom;
    /** La date de naissance de la personne. */
    Date _dateDeNaissance;

// METHODES PUBLIQUES
public:
    //-----------------------------
    // Constructeurs/destructeurs
    //-----------------------------
    /** Constructeur par defaut. */
    Personne();
    /** Constructeur par initialisation. */
    Personne(const string & iPrenom, const string & iNom, const Date & iDate);
    /** Constructeur par copie. */
    Personne(const Personne & p);
    /** Destructeur. */
    ~Personne();

    //-----------------------------
    // Selecteurs
    //-----------------------------
    /**
     Retourne une référence constante sur la string (ce qui évite la copie)
    L'autre solution serait de retourner une copie du nom de la personne*/
    const string & getNom() const;

    const string & getPrenom() const;

    const Date & getDateDeNaissance() const;

    /** Modification du nom. */
    void setNom(const string & iNom);
    /** Modification du prenom. */
    void setPrenom(const string & iPrenom);
    /** Modification de la date de naissance. */
    void setDateDeNaissance(const Date&);

    //-----------------------------
    // Methodes
    //-----------------------------
    /** output des caracteristiques de la personne (flux par defaut : l'écran) */
    virtual void output(ostream & flux=cout) const;
    /** input d'une personne (flux par defaut : le clavier)*/
    virtual void input(istream & flux=cin);

    virtual Personne * clone() const =0;


    //-----------------------------
    // Surcharges d'operateurs
    //-----------------------------
    /** Operateur d'affectation. */
    Personne& operator = (const Personne & p);
    /** Test d'"inferiorite" avec une autre personne, selon le nom, puis le prenom, puis la date de naissance. */
    bool operator < (const Personne & p) const;
    /** Test de "superiorite" avec une autre personne, selon le nom, puis le prenom, puis la date de naissance. */
    bool operator > (const Personne & p) const;
    /** Test d'"egalite" entre deux personnes : nom, prenom et date de naissance identiques. */
    bool operator == (const Personne & p) const;

    // operateurs de flux
    //-------------------
    /** Saisie d'une personne par l'entree standard. */
    friend istream & operator >> (istream & flux, Personne & p);
    /** Affichage d'une personne par la sortie standard. */
    friend ostream & operator << (ostream & flux, const Personne & p);


};



#endif // ifndef __PERSONNE_H__

