//
// C++ Implementation: population
//
// Description:
//
//
// Author:  <>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "population.h"

Population::Population()
{
  _taille = 0;
  _contenu = NULL;
}



Population::Population(int taille){
  if (taille != 0){
    _taille = taille;
    _contenu = new Personne*[_taille];
    for (int i=0; i<_taille; i++){
      // on créé une personne par défaut
      _contenu[i] = creerPersonne();
    }
  }
  else{
    _contenu = NULL;
    _taille = 0;
  }
}



Population::Population(const Population & p){
  _taille = 0;
  _contenu = NULL;
  setContenu(p.getContenu(), p.getTaille());
}



int Population::getTaille()const{
  return _taille;
}



Population::~Population()
{
  if (_contenu != NULL){
    for (int i=0; i<_taille; i++){
      if (_contenu[i] != NULL){
        delete _contenu[i];
        _contenu[i] = NULL;
      }
    }
    delete [] _contenu;
  }
}




const Personne * const * Population::getContenu() const{
  return _contenu;
}


void Population::setContenu(const Personne * const * t, int taille){
  if (_taille!=0){
    for (int i=0; i<_taille; i++){
      delete _contenu[i];
    }
    _taille=0;
    delete [] _contenu;
  }
  _taille = taille;
  _contenu = new Personne*[_taille];
  for (int i=0; i<_taille; i++){
    if (t[i]){
      _contenu[i] = t[i]->clone();
    }
    else{
      // pointeur NULL, on décide de créer une Personne par défaut
      _contenu[i] = creerPersonne();
    }
  }
}



const Personne * Population::getPersonne(int i) const{
  const Personne * res = NULL;
  if (i>=0 && i<_taille){
    res = _contenu[i];
  }
  return res;
}


void Population::output(ostream & flux) const{
  if (_contenu != NULL){
    int compt = 0;
    for (int i=0; i<_taille; i++){
      if (_contenu[i] != NULL){
        compt++;
        flux<<"Personne "<<compt<<" : "<<endl;
        _contenu[i]->output(flux); // méthode virtuelle
        flux<<endl<<endl;
      }
    }
  }
}

void Population::input(istream & flux){
  if (_contenu != NULL){
    for (int i=0; i<_taille; i++){
      delete _contenu[i];
    }
    delete [] _contenu;
  }
  _taille = 0;
  while (_taille <= 0){
    if (flux==cin){
      cout<<"Entrer le nombre de personnes"<<endl;
    }
    flux>>_taille;
  }
  _contenu = new Personne*[_taille];
  for (int i=0; i<_taille; i++){
    char r='z';
    while (r!='e' && r!='a'){
      if (flux==cin){
        cout<<"Adulte (a) ou Enfant (e) ?"<<endl;
      }
      flux>>r;
    }
    if (r=='e'){
      _contenu[i] = new Enfant();
    }
    else{
      _contenu[i] = new Adulte();
    }
    _contenu[i]->input(flux);
  }
}




istream& operator >> (istream& is, Population& p){
  p.input(is);
  return is;
}


ostream& operator << (ostream& os, const Population& p){
  p.output(os);
  return os;
}




